@@ -21,6 +21,7 @@
 
 #include "system.h"
 #include "dev-ino.h"
+#include "die.h"
 #include "error.h"
 #include "ignore-value.h"
 #include "quote.h"
@@ -512,14 +513,14 @@ main (int argc, char **argv)
       if (bit_flags == FTS_PHYSICAL)
         {
           if (dereference == 1)
-            error (EXIT_FAILURE, 0,
-                   _("-R --dereference requires either -H or -L"));
+            die (EXIT_FAILURE, 0,
+                 _("-R --dereference requires either -H or -L"));
           affect_symlink_referent = false;
         }
       else
         {
           if (dereference == 0)
-            error (EXIT_FAILURE, 0, _("-R -h requires -P"));
+            die (EXIT_FAILURE, 0, _("-R -h requires -P"));
           affect_symlink_referent = true;
         }
     }
@@ -543,8 +544,8 @@ main (int argc, char **argv)
       char *ref_context = NULL;
 
       if (getfilecon (reference_file, &ref_context) < 0)
-        error (EXIT_FAILURE, errno, _("failed to get security context of %s"),
-               quoteaf (reference_file));
+        die (EXIT_FAILURE, errno, _("failed to get security context of %s"),
+             quoteaf (reference_file));
 
       specified_context = ref_context;
     }
@@ -557,8 +558,8 @@ main (int argc, char **argv)
     {
       specified_context = argv[optind++];
       if (security_check_context (se_const (specified_context)) < 0)
-        error (EXIT_FAILURE, errno, _("invalid context: %s"),
-               quote (specified_context));
+        die (EXIT_FAILURE, errno, _("invalid context: %s"),
+             quote (specified_context));
     }
 
   if (reference_file && component_specified)
@@ -572,8 +573,8 @@ main (int argc, char **argv)
       static struct root_dev_ino dev_ino_buf;
       root_dev_ino = get_root_dev_ino (&dev_ino_buf);
       if (root_dev_ino == NULL)
-        error (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
-               quoteaf ("/"));
+        die (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
+             quoteaf ("/"));
     }
   else
     {
