@@ -23,6 +23,7 @@
 
 #include "system.h"
 #include "chown-core.h"
+#include "die.h"
 #include "error.h"
 #include "fts_.h"
 #include "quote.h"
@@ -230,7 +231,7 @@ main (int argc, char **argv)
                                              &required_uid, &required_gid,
                                              NULL, NULL);
             if (e)
-              error (EXIT_FAILURE, 0, "%s: %s", e, quote (optarg));
+              die (EXIT_FAILURE, 0, "%s: %s", e, quote (optarg));
             break;
           }
 
@@ -262,8 +263,8 @@ main (int argc, char **argv)
       if (bit_flags == FTS_PHYSICAL)
         {
           if (dereference == 1)
-            error (EXIT_FAILURE, 0,
-                   _("-R --dereference requires either -H or -L"));
+            die (EXIT_FAILURE, 0,
+                 _("-R --dereference requires either -H or -L"));
           dereference = 0;
         }
     }
@@ -286,8 +287,8 @@ main (int argc, char **argv)
     {
       struct stat ref_stats;
       if (stat (reference_file, &ref_stats))
-        error (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
-               quoteaf (reference_file));
+        die (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
+             quoteaf (reference_file));
 
       uid = ref_stats.st_uid;
       gid = ref_stats.st_gid;
@@ -299,7 +300,7 @@ main (int argc, char **argv)
       const char *e = parse_user_spec (argv[optind], &uid, &gid,
                                        &chopt.user_name, &chopt.group_name);
       if (e)
-        error (EXIT_FAILURE, 0, "%s: %s", e, quote (argv[optind]));
+        die (EXIT_FAILURE, 0, "%s: %s", e, quote (argv[optind]));
 
       /* If a group is specified but no user, set the user name to the
          empty string so that diagnostics say "ownership :GROUP"
@@ -315,8 +316,8 @@ main (int argc, char **argv)
       static struct root_dev_ino dev_ino_buf;
       chopt.root_dev_ino = get_root_dev_ino (&dev_ino_buf);
       if (chopt.root_dev_ino == NULL)
-        error (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
-               quoteaf ("/"));
+        die (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
+             quoteaf ("/"));
     }
 
   bit_flags |= FTS_DEFER_STAT;
